<?php

require_once('db_lib.php'); 
require_once('PostedObjectCache.php'); 
require_once('../vendor/autoload.php'); 

use Aws\Sqs\SqsClient;

class SqsObjectCache extends PostedObjectCache
{
    private $environment;
    public $sqsClient;
    private $outputQueue;
    private $inputQueue;
    
    
    var $currentMessageTags = [];
    
    private static $AWS_CREDENTIALS =
    [
        'dev' => 
        [
            'region' => 'eu-west-2',
            'version' => 'latest',
            'credentials' => 
            [
                //SQSuser credentials, a user specifically set up only for SQS
                'key' => 'AKIAJEPTNRHRDNIY2KDA',             
                'secret' => 'password'
            ]
        ],
        'prod' => 
        [
        ]
    ];
    //these aren't complete, space in place to put production queues
    private static $AWS_QUEUE_URLS =
    [
        'dev' =>
        [
            'gnip' => 'https://sqs.eu-west-2.amazonaws.com/697738154271/GnipParsingQueueTest',
            'parsed' => 'https://sqs.eu-west-2.amazonaws.com/697738154271/ParsedQueueTest'
        ],
        'prod' =>
        [
        ]
    ];
    
    function __construct($postType="tweets",$targetDb="elastic",$dbWrapper=null,$environment=null,$ErrorLogger=null){
        $this->environment = $environment;
        $credentials = self::$AWS_CREDENTIALS[ $environment ];
        //this ideally shouldn't be hardcoded here
        $this->inputQueue = self::$AWS_QUEUE_URLS[$this->environment]['gnip'];
        $this->outputQueue = self::$AWS_QUEUE_URLS[$this->environment]['parsed'];

        $this->sqsClient = SqsClient::factory($credentials);
        
    }
    //currently unused in blurrt parser, useful to put raw data onto queue directly
    //currently hardcoded to place onto the output queue
    public function addToQueue($data){
        $this->sqsClient->sendMessage(array(
            'QueueUrl' => $this->outputQueue,
            //'QueueUrl' => $this->inputQueue,
            'MessageBody' => $data
        ));
    }
    
    public function save($Blurrt){
        $data = json_encode( $Blurrt->getPayload() );
        
        $this->sqsClient->sendMessage(array(
            'QueueUrl' => $this->outputQueue,
            'MessageBody' => $data
        ));
    }

    public function getDataBatch($batchSize=10, $strictBatchSize=false){
        
        $output = $this->sqsClient->receiveMessage(
            array(
                'QueueUrl' => $this->inputQueue,
                'MaxNumberOfMessages' => $batchSize
            ));
            
        $outputArray = array();
        foreach($output['Messages'] as $messages){
            $this->currentMessageTags[] = ['Id'=> $messages['MessageId'] ,'ReceiptHandle'=>$messages['ReceiptHandle']];
            $outputArray[] = ['json' => $messages['Body']];
        }
        //print_r($this->currentMessageTags);
        
        return $outputArray;
    }
    //Amazon SQS has a timeout system, if a message isn't deleted it can be pulled in again after the timeout
    //this method therefore deletes the returned message from getting the data batch
    //input = array of arrays contain id, key pairs
    public function markDataBatchProcessed(){
        if(count($this->currentMessageTags)>0){
            $output = $this->sqsClient->deleteMessageBatch([
                'Entries' => $this->currentMessageTags, 
                'QueueUrl' => $this->inputQueue
            ]);
        }
        $this->currentMessageTags = [];
    }
    //SQS doesn't need marking as unprocessed because it does that automatically
    //If the databatch isn't marked at processed it's requeued
    public function markDataBatchUnProcessed(){
        $this->currentMessageTags = [];
    }
    
    public function getAttribute($queue){
        return $this->sqsClient->getQueueAttributes([
            'QueueUrl' => self::$AWS_QUEUE_URLS[$this->environment][$queue],
            'AttributeNames' => ['All']
        ])['Attributes'];
    }

}
