<?php

require_once('db_lib.php'); 
require_once('PostedObjectCache.php'); 
require_once('../vendor/autoload.php'); 

use Aws\Sqs\SqsClient;

class SqsObjectCache extends PostedObjectCache
{
    private $environment;
    public $sqsClient;
    private $outputQueue;
    private $inputQueue;
    
    
    var $currentMessageTags = [];
    
    private static $AWS_CREDENTIALS =
    [
        'dev' => 
        [
            'region' => 'eu-west-2',
            'version' => 'latest',
            'credentials' => 
            [
                //SQSuser credentials, a user specifically set up only for SQS
                'key' => 'AKIAJEPTNRHRDNIY2KDA',             
                'secret' => 'password'
            ]
        ],
        'prod' => 
        [
        ]
    ];
    //these aren't complete, space in place to put production queues
    private static $AWS_QUEUE_URLS =
    [
        'dev' =>
        [
            'gnip' => 'https://sqs.eu-west-2.amazonaws.com/697738154271/GnipParsingQueueTest',
            'parsed' => 'https://sqs.eu-west-2.amazonaws.com/697738154271/ParsedQueueTest'
        ],
        'prod' =>
        [
        ]
    ];
    
    function __construct($postType="tweets",$targetDb="elastic",$dbWrapper=null,$environment=null,$ErrorLogger=null){
        $this->environment = $environment;
        $credentials = self::$AWS_CREDENTIALS[ $environment ];
        //this ideally shouldn't be hardcoded here
        $this->inputQueue = self::$AWS_QUEUE_URLS[$this->environment]['gnip'];
        $this->outputQueue = self::$AWS_QUEUE_URLS[$this->environment]['parsed'];

        $this->sqsClient = SqsClient::factory($credentials);
        
    }
    //currently unused in blurrt parser, useful to put raw data onto queue directly
    //currently hardcoded to place onto the output queue
    public function addToQueue($data){
        $this->sqsClient->sendMessage(array(
            //'QueueUrl' => $this->outputQueue,
            'QueueUrl' => $this->inputQueue,
            'MessageBody' => $data
        ));
    }
    
    public function save($Blurrt){
        $data = json_encode( $Blurrt->getPayload() );
        
        $this->sqsClient->sendMessage(array(
            'QueueUrl' => $this->outputQueue,
            'MessageBody' => $data
        ));
    }

    public function getDataBatch($batchSize=10, $strictBatchSize=false){
        
        $output = $this->sqsClient->receiveMessage(
            array(
                'QueueUrl' => $this->inputQueue,
                'MaxNumberOfMessages' => $batchSize
            ));
            
        $outputArray = array();
        foreach($output['Messages'] as $messages){
            $this->currentMessageTags[] = ['Id'=> $messages['MessageId'] ,'ReceiptHandle'=>$messages['ReceiptHandle']];
            $outputArray[] = ['json' => $messages['Body']];
        }
        //print_r($this->currentMessageTags);
        
        return $outputArray;
    }
    //Amazon SQS has a timeout system, if a message isn't deleted it can be pulled in again after the timeout
    //this method therefore deletes the returned message from getting the data batch
    //input = array of arrays contain id, key pairs
    public function markDataBatchProcessed(){
        if(count($this->currentMessageTags)>0){
            $output = $this->sqsClient->deleteMessageBatch([
                'Entries' => $this->currentMessageTags, 
                'QueueUrl' => $this->inputQueue
            ]);
        }
        $this->currentMessageTags = [];
    }
    //SQS doesn't need marking as unprocessed because it does that automatically
    //If the databatch isn't marked at processed it's requeued
    public function markDataBatchUnProcessed(){
        $this->currentMessageTags = [];
    }
    
    public function getAttribute($queue){
        return $this->sqsClient->getQueueAttributes([
            'QueueUrl' => self::$AWS_QUEUE_URLS[$this->environment][$queue],
            'AttributeNames' => ['All']
        ])['Attributes'];
    }

}



$asdf = new SqsObjectCache(null, null, null, 'dev');

for($i=0;$i<1000;$i++){
    $asdf->addToQueue('{"created_at":"Thu Mar 23 17:08:09 +0000 2017","id":'.(String)(844958961069547500+$i).',"id_str":"844958961069547521","text":"#MotoGP followed by #F1.......from now til'.date('c'). ' Sunday......#DoNotDisturb","source":"<a href=\"http://twitter.com\" rel=\"nofollow\">Twitter Web Client</a>","truncated":false,"in_reply_to_status_id":null,"in_reply_to_status_id_str":null,"in_reply_to_user_id":null,"in_reply_to_user_id_str":null,"in_reply_to_screen_name":null,"user":{"id":16704454,"id_str":"16704454","name":"bimblelass/Karen","screen_name":"bimblelass","location":"Leigh NW England","url":null,"description":"1 life lots of loves - my family and pet pooches; @MCFC (ST block137); #F1- #LH;#McLaren;#MotoGP-#VR/CC/NH/BS. Spiritual.","translator_type":"none","protected":false,"verified":false,"followers_count":1402,"friends_count":1090,"listed_count":63,"favourites_count":13540,"statuses_count":19092,"created_at":"Sun Oct 12 04:23:24 +0000 2008","utc_offset":0,"time_zone":"London","geo_enabled":true,"lang":"en","contributors_enabled":false,"is_translator":false,"profile_background_color":"642D8B","profile_background_image_url":"http://pbs.twimg.com/profile_background_images/434651370835030016/RqQQO6Cv.jpeg","profile_background_image_url_https":"https://pbs.twimg.com/profile_background_images/434651370835030016/RqQQO6Cv.jpeg","profile_background_tile":true,"profile_link_color":"89C9FA","profile_sidebar_border_color":"FFFFFF","profile_sidebar_fill_color":"7AC3EE","profile_text_color":"3D1957","profile_use_background_image":true,"profile_image_url":"http://pbs.twimg.com/profile_images/802900218220003328/pamMwZi__normal.jpg","profile_image_url_https":"https://pbs.twimg.com/profile_images/802900218220003328/pamMwZi__normal.jpg","profile_banner_url":"https://pbs.twimg.com/profile_banners/16704454/1398280620","default_profile":false,"default_profile_image":false,"following":null,"follow_request_sent":null,"notifications":null},"geo":null,"coordinates":null,"place":null,"contributors":null,"is_quote_status":false,"quote_count":0,"reply_count":0,"retweet_count":0,"favorite_count":0,"entities":{"hashtags":[{"text":"MotoGP","indices":[0,7]},{"text":"F1","indices":[20,23]},{"text":"DoNotDisturb","indices":[55,68]}],"urls":[],"user_mentions":[],"symbols":[]},"favorited":false,"retweeted":false,"filter_level":"low","lang":"en","timestamp_ms":"1490288889554","matching_rules":[{"tag":"3333673","id":843762953522499600}]}');
}
print_r("done\n");

